package simulation_Lab12Version;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics2D;
//import java.awt.Image;
//import java.io.File;
//import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Vector;

//import javax.imageio.ImageIO;

import sprites.FlowerSprite;
import sprites.BeetleShooter;
import sprites.ProjectileSprite;
import sprites.ShooterSprite;
import sprites.Sprite;

/**
 * This class encapsulates a simple shooter game world.
 * 
 * @author mb, modified by YL
 * 
 */
public class AnimationModel_Lab12 implements Model
{

	private List<ModelListener> listeners;
	private Dimension worldDimension = UtilityClass.getQuarterOfScreen();

	private Collection<Sprite> theTargets;

	private ShooterSprite theShooter;
	private Collection<ProjectileSprite> theProjectiles;

	private final int TARGET_COUNT = 10;

	// private Image bkgdImg;

	/**
	 * Constructs an animation model.
	 */
	public AnimationModel_Lab12()
	{
		listeners = new Vector<ModelListener>();
		theTargets = new ArrayList<Sprite>();
		theProjectiles = new ArrayList<ProjectileSprite>();
		theShooter = new BeetleShooter(getWorldWidth(), getWorldHeight());

		for (int i = 0; i < TARGET_COUNT; i++)
		{

			Sprite theSprite = new FlowerSprite(getWorldWidth(),
					getWorldHeight());

			// create a slight offset so that sprites are not positioned upon
			// one another
			int offset = i * 75;

			// reposition to top of screen
			theSprite.reposition(getWorldWidth() / 3 + offset,
					getWorldHeight() / 8);

			theTargets.add(theSprite);
		}

		// try {
		// // the following method will block until the image is fully
		// // loaded (the same is not true for the getImage method)
		// bkgdImg = ImageIO.read(new File("assets/275114.jpg"));
		// } catch (IOException e) {
		// e.printStackTrace();
		// System.exit(0);
		// }
	}

	public int getWorldWidth()
	{
		return worldDimension.width;
	}

	public void setWorldWidth(int worldWidth)
	{
		this.worldDimension = new Dimension(worldWidth, worldDimension.height);
	}

	public int getWorldHeight()
	{
		return worldDimension.height;
	}

	public void setWorldHeight(int worldHeight)
	{
		this.worldDimension = new Dimension(worldDimension.width, worldHeight);
	}

	public void notifyModelHasChanged()
	{
		for (ModelListener listener : listeners)
		{
			listener.changed();
		}
	}

	public void advanceNextState()
	{
		for (Sprite s : theTargets)
		{
			s.moveRight();
		}

		boolean addNewTarget = false;
		for (ProjectileSprite s : theProjectiles)
		{
			s.move();
			// now check whether this projectile has hit any targets
			for (Sprite target : theTargets)
			{
				// if (!projectile.isAlive())
				// toBeRemoved.add(ps);
				if (target.isCollided(s))
				{
					target.setExploded();
					addNewTarget = true;
				}
			}
		}

		// for (ProjectileSprite ps : toBeRemoved) {
		// theSprites.remove(ps);
		// }

		if (addNewTarget)
		{
			Sprite theSprite = new FlowerSprite(getWorldWidth(),
					getWorldHeight());
			// reposition to center
			theSprite.reposition(
					UtilityClass.getRandomValue(0, getWorldWidth()),
					UtilityClass.getRandomValue(0, getWorldHeight()));
			theTargets.add(theSprite);

		}

		notifyModelHasChanged();
	}

	public void addListener(ModelListener listener)
	{
		listeners.add(listener);
	}

	public void drawCurrentState(Graphics2D g)
	{
		for (Sprite s : theTargets)
		{
			s.draw(g);
		}
		for (ProjectileSprite s : theProjectiles)
		{
			s.draw(g);
		}

		// System.out.println("notify model changed");
		if (theShooter != null)
		{
			theShooter.draw(g);
		}
	}

	public Dimension getWorldDimension()
	{
		return this.worldDimension;
	}

	public void repositionAllSpritesToCentre()
	{
		for (Sprite s : theTargets)
		{
			s.reposition(getWorldWidth() / 2, getWorldHeight() / 2);
		}
		notifyModelHasChanged();
	}

	public void drawSceneBackground(Graphics2D g)
	{
		// clear the screen
		g.setPaint(Color.WHITE);
		g.fillRect(0, 0, getWorldWidth(), getWorldHeight());
		// g.drawImage(bkgdImg, 0, 0, getWorldWidth(), getWorldHeight(), null);
	}

	/**
	 * Adds a new sprite to the centre of the world
	 */
	public void addSprite()
	{

	}

	public void moveShooterRight()
	{
		theShooter.moveRight();
		notifyModelHasChanged();
	}

	public void moveShooterLeft()
	{
		theShooter.moveLeft();
		notifyModelHasChanged();
	}

	public void moveShooterDown()
	{
		theShooter.moveDown();
		notifyModelHasChanged();
	}

	public void moveShooterUp()
	{
		theShooter.moveUp();
		notifyModelHasChanged();
	}

	public void fireShooter()
	{
		ProjectileSprite newProjectile = theShooter.fire();
		// System.out.println("Fired");
		theProjectiles.add(newProjectile);
		// theSpriteCollection.add(theProjectile);

		notifyModelHasChanged();
	}

	public void rotateShooterRight()
	{
		theShooter.rotateRight();
	}

	public void rotateShooterLeft()
	{
		theShooter.rotateLeft();
	}

}
