package simulation_Lab12Version;

import java.awt.event.ActionListener;

import javax.swing.Timer;

/**
 * This class encapsulates services for launching a new thread and for starting
 * a process on that thread that dispatches events at the specified rate.
 * Objects of this type can support a single observer (additional observers
 * cannot be installed).
 * 
 * @author mb
 * 
 */
public class Observee {

	private Timer frameAdvancerTimer;

	/**
	 * Instantiates an object that dispatches events and that can be observed
	 * (an observee).
	 * 
	 * @param numberEventsToBeDispatchedPerSecond
	 *            an int that is strictly greater than zero, representing the
	 *            number of events to be dispatched per second
	 * @param theObserver
	 *            the object that will be listening for the events dispatched by
	 *            this object
	 */
	public Observee(int numberEventsToBeDispatchedPerSecond,
			ActionListener theObserver) {
		// here we determine how much time each frame should spend on-screen
		final int NUM_MSEC_PER_SEC = 1000;
		int msecPerFrame = NUM_MSEC_PER_SEC
				/ numberEventsToBeDispatchedPerSecond;

		frameAdvancerTimer = new Timer(msecPerFrame, theObserver);
	}

	/**
	 * Launches the thread.
	 */
	public void start() {
		frameAdvancerTimer.start();
	}

	/**
	 * Determines whether this Observee is currently running and dispatching
	 * events.
	 * 
	 * @return as described above
	 */
	public boolean isRunning() {
		return frameAdvancerTimer.isRunning();
	}

}
