package sprites;

import java.awt.Point;

import com.sun.corba.se.spi.legacy.connection.GetEndPointInfoAgainException;

public abstract class ProjectileSprite extends Sprite {

	private final int LIFETIME = 50;
	private int lifeTimeInFrames = LIFETIME;

	private int xDisplacement;
	private int yDisplacement;
	private double angularDisplacement = 0;
	private boolean isFresh;

	private boolean isLive;

	/**
	 * 
	 * @param widthOfDrawingRegion
	 * @param heightOfDrawingRegion
	 * @param xDisplacement
	 *            expresses the x-component of the movement of the projectile
	 * @param yDisplacement
	 *            expresses the y-component of the movement of the projectile
	 */
	protected ProjectileSprite(int widthOfDrawingRegion,
			int heightOfDrawingRegion, int xDisplacement, int yDisplacement) {
		super(widthOfDrawingRegion, heightOfDrawingRegion);
		setxDisplacement(xDisplacement);
		setyDisplacement(yDisplacement);

		double angDisplacement = 0;
		if (yDisplacement < 0)
			angDisplacement = Math.atan(-getxDisplacement()
					/ (double) getyDisplacement());
		else
			angDisplacement = Math.atan(-getxDisplacement()
					/ (double) getyDisplacement())
					+ Math.PI;
		setAngularDisplacement(angDisplacement);
	}

	public int getxDisplacement() {
		return xDisplacement;
	}

	public void setxDisplacement(int xDisplacement) {
		this.xDisplacement = xDisplacement;
	}

	public int getyDisplacement() {
		return yDisplacement;
	}

	public void setyDisplacement(int yDisplacement) {
		this.yDisplacement = yDisplacement;
	}

	public double getAngularDisplacement() {
		return angularDisplacement;
	}

	public void setAngularDisplacement(double angularDisplacement) {
		this.angularDisplacement = angularDisplacement;
	}

	public int getLifeTimeInFrames() {
		return lifeTimeInFrames;
	}

	public boolean isFresh() {
		return isFresh;
	}

	public void setFresh(boolean isFresh) {
		this.isFresh = isFresh;
	}

	/*
	 * Returns whether this ammo is live and should be drawn. (When ammo travels
	 * off-screen, stop drawing it)
	 */
	public boolean isLive() {
		return isLive;
	}

	/*
	 * Setter for whether this ammo is live and should be drawn. (When ammo
	 * travels off-screen, stop drawing it)
	 */
	public void setLive(boolean isLive) {
		this.isLive = isLive;
	}

	public void move() {
		getTheBoundingShape().translate(xDisplacement, yDisplacement);
		killOffIfOffScreen();
		killOffIfLifeTimeHasExpired();
		updateLifeTime();
	}

	private void updateLifeTime() {
		// decrement this projectile's lifetime
		lifeTimeInFrames--;
		if (lifeTimeInFrames < LIFETIME / 2)
			isFresh = false;
	}

	private void killOffIfLifeTimeHasExpired() {
		// check if projectile's lifetime has elapsed
		if (lifeTimeInFrames == 0) {
			isLive = false;
		}
	}

	private void killOffIfOffScreen() {
		// check if the projectile has moved off-screen

		// boolean isOffScreenHorizontal = this.getCurrentX() > worldWidth
		// || this.getCurrentX() < 0;
		// boolean isOffScreenVertical = this.getCurrentY() > worldHeight
		// || this.getCurrentY() < 0;
		// if (isOffScreenHorizontal || isOffScreenVertical) {
		// isLive = false;
		// }
	}

}
